/*
 * ATmega328P_InputCapture_Timer1_Test.cpp
 *
 * Created: 19.08.2020 15:11:04
 * Author : Micha
 *
 * Atmel ATmega328P und eine LED an PortC.0 und PortC.1
 * Test einer Zeitmessung eines Rechtecksignals an PortB.0 / ICP1
 * Ausgabe LED blinken bei jedem Timer1_Capture_Interrupt  an PortC.0 (besser mit Scope)
 * Ausgabe LED blinken bei jedem 1000.ten Interrupt an PortC.1 
 *
 * Messbereich bei 1 MHz Timertakt ca. 35 Hz bis 15000 Hz
 * Messbereich bei 125 kHz Timertakt: ca. 2 bis 500 Hz
 *
 * Fuses (AtMega328P): Ext.: 0xFF, HIGH=0xD1, LOW=0xE2
 
 * Alternative Hardware:
 * AtMega1284P, LED an PC0, Rechtecksignal an ICP1 (PD6 IC-Pin 20)
 * Textausgabe auf UART0
 * 
 */ 

/* nderung bei den Parametern des Linkers
  Zur Benutzung von sprintf() und float / double-Variablen - nderungen beim Linker:
- Men Projekt, Properties zu diesem Projekt aufrufen
- unter Linker / General die Option  "Use vprintf-library" anklicken
- unter Linker / Miscellaneous in das Feld eintragen: -lprintf_flt
- nun kann man sprintf() auch benutzen mit float und double
- Quelle: https://startingelectronics.org/articles/atmel-AVR-8-bit/print-float-atmel-studio-7/
*/

// Frequenz = 1 / Periode
// Hertz = Taktfrequenz des Timers in Hertz / gemessenen Wert
					
// Ist der Timertakt 1 MHz (8 MHz CPU-Takt / 8 (Prescaler))
// entspricht ein gemessener Wert von 177 damit 177 s
//
// Messbereich in dieser Einstellung: ca. 35 Hz bis 15000 Hz
// Genauigkeit ist im unteren Bereich hoch ( 100 Hz +1 1 Hz) und nimmt nach oben zunehmend ab
// bei 25 kHz ist das Resultat etwa 29 kHz.
// Bei hheren Frequenzen geht die Verarbeitungszeit des Interrupt und
// der Berechnungen immer mehr in das Ergebnis ein und macht es ungenauer
					
// bei 125000 Hz Timertakt ( 8 MHz CPU-Takt / 64):
//double hertz = (double) 125000 / werte[i];

// bei 1 MHz Timertakt ( 8MHz CPU-Takt / 8):
// uint16_t hertz = 1000000 / werte[i];

// bei 8 MHz Timertakt:
// uint16_t hertz = 8000000 / werte[i];

#ifndef F_CPU
#define F_CPU 8000000UL								// Takt 8 MHz interner RC-Oszillator
#endif

#include <avr/io.h>
//#include <avr/sleep.h>
#include <avr/interrupt.h>
//#include <util/delay.h>
#include <stdio.h>
#include <stdlib.h>

// Bitkram
//PORTA  |= (1 << bit);					// set Bit  in PortA
//PORTA  &= ~(1 << bit);				//Reset Bit in PortA
//PORTB &= ~((1 << 0) | (1 << 2));		// lscht Bit 0 und 2 in PORTB
//PORTB ^= (1 << PB0);					// toggles Bit 0 in PORTB

#define speichertiefe 20 

// Variablen
volatile uint8_t cnt = 0;				// Interrupt-Zhler
uint16_t werte[speichertiefe];			// gemessene Werte
volatile uint8_t indx = 0;
double  hz;



//----------------------------------------------------------------------------------------------------------
// USART:
//
#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)
char s[20];
char rx_value;
char dummy_char;

void USART_Init(void)
{
   DDRD  |= (1 << 1);					// set Bit PD1 -> TxD = Ausgang (AtMega328P,Atmega1284P)
   // Set baud rate
   UBRR0L = BAUD_PRESCALE;// Load lower 8-bits into the low byte of the UBRR register
   UBRR0H = (BAUD_PRESCALE >> 8); 
	/* Load upper 8-bits into the high byte of the UBRR register
    Default frame format is 8 data bits, no parity, 1 stop bit
  to change use UCSRC, see AVR datasheet */ 

  // Enable receiver and transmitter and receive complete interrupt 
  UCSR0B = ((1<<TXEN0) | (1<<RXEN0) | (1<<RXCIE0));
}

ISR(USART_RX_vect)
{
	rx_value = UDR0;             //dummy read
}

void UARTsendString(char data[])   //sendet Char-Array
{
	uint8_t i = 0;
	while(!(UCSR0A & (1<<UDRE0)))
	{
		;//Do nothing
	}
	
	while (data[i] != '\0')
	{
		UDR0 = data[i];
		while ( !( UCSR0A & (1 << UDRE0)) )
		;
		i++;
	}
}

 // ein Zeichen senden
 void UARTWriteChar(char data)
{
	//Wait until the transmitter is ready

	while(!(UCSR0A & (1<<UDRE0)))
	{
		;//nichts machen, nur warten
	}

	//Now write the data to USART buffer

	UDR0=data;
}

// Wait until a byte has been received and return received data
uint8_t UART_ReceiveByte(){
	while((UCSR0A &(1<<RXC0)) == 0);
	return UDR0;
}

// ein Zeichen lesen
char UARTReadChar()
{
	//Wait until a data is available

	while(!(UCSR0A & (1<<RXC0)))
	{
		//Do nothing
	}

	//Now USART has got data from host
	//and is available in buffer

	return UDR0;
}
//----------------------------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------------------------
// Timer1 im Capture Modus zur Erfassung eines Rechtecksignals
//
// Eingangspin ist ICP1 (PortB.0 beim AtMega328P, PD6 beim ATMega1284P)
void timer1_init()
{
	cli();
	TCNT1 = 0;										// Zhler des Timer1 auf Null
	TCCR1A = 0;										// keine Einstellungen in TCCR1A
	
	// Timer1-Takt soll dem des CPU-Taktes entsprechen (kein Prescaler)
	TCCR1B &= ~(1 << CS12);							// Timer1-Takt = CPU-Takt:
	TCCR1B &= ~(1 << CS11);							// CS12 = 0, CS11 = 0, CS10 = 1
	TCCR1B |= (1 << CS10);							// also hier Timer1 mit 8 MHz
	
	// Timer1-Takt ist CPU-Takt / 8:		CS12 = 0 / CS11 = 1 / CS10 = 0
	//TCCR1B &= ~(1 << CS12);							// CS12 = 0
	//TCCR1B |= (1 << CS11);							// CS11 = 1
	//TCCR1B &= ~(1 << CS10);							// CS10 = 0
	
	// Timer1-Takt ist CPU-Takt / 64:		CS12 = 0 / CS11 = 1 / CS10 = 1
	//TCCR1B &= ~(1 << CS12);							// CS12 = 0
	//TCCR1B |= (1 << CS11);							// CS11 = 1
	//TCCR1B |= (1 << CS10);							// CS12 = 1
	
	// Noise Canceller (Strunterdrckung ein/aus)
	TCCR1B |= (1 << ICNC1);							// ICNC1 = 1 Enable Noise Canceller 
	//TCCR1B &= ~(1 << ICNC1);							// ICNC1 = 0 Disable Noise Canceller 
	
	// Interrupt bei steigender oder fallender Flanke
	TCCR1B |= (1 << ICES1);							// ICES1 = 1, Interrupt bei steigende Flanke 
	//TCCR1B &= ~(1 << ICES1);						// ICES1 = 0, Interrupt bei fallende Flanke
	
	// Interrupt einschalten
	TIMSK1	|= (1 << ICIE1);						// Input Capture Event Interrupt enable
	
	// Globale Interrupts einschalten
	sei();
}

void timer1_ein()
{
	//cli();
	TIMSK1	|= (1 << ICIE1);						// enable Input Capture Event Interrupt
	//sei();
}

void timer1_aus()
{
	//cli();
	TIMSK1	&= ~(1 << ICIE1);						// disable Input Capture Event Interrupt 
	//sei();
}

ISR(TIMER1_CAPT_vect)
{
	//cli();
	werte[indx++] = ICR1;						// gemessener Zhlerstand merken
	TCNT1 = 0;									// Zhler des Timers1 zurckstellen
	cnt++;										// Interrupt Zhler + 1
	if (indx >= speichertiefe) { indx = 0; }
	//PORTC ^= (1 << 0);							// toggles PORTC.0 -> LED 
	//sei();
}
//----------------------------------------------------------------------------------------------------------


void Setup()
{
	DDRC = 0x03;									// PortC.0 und PortC.1 = Output fr zwei LEDs 
	USART_Init();
	timer1_init();
	cnt = 0;
	indx = 0;
	sei();
	
	sprintf(s, " - Start - \r\n");
	UARTsendString(s);
}


double frequenzerkennung()
{
	double durchschnitt;
	double summe = 0;
	double hertz = 0;
	
	indx = 0;
	cnt = 0;
	timer1_ein();
	while(cnt < speichertiefe) { ; }
	timer1_aus();
	
	if(cnt >= speichertiefe)									// x Mal Interrupt erkannt, dann gemessene Werte anzeigen
	{
		cnt = 0;
		for(int i=0; i < speichertiefe; i++)					
		{
			//sprintf(s, "werte[] = %d %d \r\n",i, werte[i]);
			//UARTsendString(s);
			summe += werte[i];
		}
		durchschnitt = summe / speichertiefe;
				
		// bei 125000 Hz Timertakt ( 8 MHz CPU-Takt / 64):
		//hertz = (double) (125000 / durchschnitt);
				
		// bei 8 MHz CPU-Takt und Prescaler 8 -> 1 MHz Timertakt:
		//hertz = (double) (1000000 / durchschnitt);
		
		// bei 8 MHz CPU-Takt und Prescaler 8 -> 1 MHz Timertakt:
		hertz = (double) (8000000 / durchschnitt);
		
		//PORTC ^= (1 << PORTC1);						// toggles PORTC.1 -> LED2
	}
	return hertz;		
}


int main(void)
{
	Setup();


    while (1) 
    {
		hz = frequenzerkennung();
		sprintf(s, "-> %.2f Hz \r\n", hz);
		UARTsendString(s);		
		
		// nur proforma, falls ein Zeichen per UART empfangen wird
		if(UCSR0A & (1<<RXC0))							// wenn Zeichen an UART
		{
			dummy_char = UARTReadChar();				// empfangenes Zeichen in den Papierkorb
		}			
    }
}

