//https://arduino.stackexchange.com/questions/38352/atmega-avr-frequency-measurement-with-timer-counter

// Frequenzzhler mit Timer 2 und Timer 1
// Hardware: AtMega328P oder AtMega1284P
//
// Timer 2: Erzeugt einen 1ms Zhler, dient als Zeitrahmen (Gate) fr die Frequenzmessung.
//			Die Zeit der Frequenzmessung gibt man im Millisekunden beim Start der Messung an.
// Timer 1: Wird vom Messsignal ber T1 mit dem Takt versorgt. Das Messsignal taktet also Timer1.
//			Messung ist auf das Hz genau.			


/*
* Bit setzen 
#define set_bit(var, bit) ((var) |= (1 << (bit)))

* Bit lschen *
#define clear_bit(var, bit) ((var) &= (unsigned)~(1 << (bit)))

* Bit togglen *
#define toggle_bit(var,bit) ((var) ^= (1 << (bit)))

* Bit abfragen *
#define bit_is_set(var, bit) ((var) & (1 << (bit)))
#define bit_is_clear(var, bit) !bit_is_set(var, bit)
*/

#include "FreqCounter.h"

volatile unsigned long FreqCounter::f_freq;
volatile unsigned char FreqCounter::f_ready;
volatile unsigned char FreqCounter::f_mlt;
volatile unsigned int FreqCounter::f_tics;
volatile unsigned int FreqCounter::f_period;

void FreqCounter::start(int ms) 
{
    f_period = ms;
	
	// use only one of the following 3 lines
	//GTCCR = (1<<TSM)|(1<<PSRASY)|(1<<PSRSYNC); // halt all timers
	//GTCCR = (1<<TSM)|(1<<PSRASY)|(0<<PSRSYNC); // halt timer2 only
	//GTCCR = (1<<TSM)|(0<<PSRASY)|(1<<PSRSYNC); // halt timer0 and timer1 only
	//GTCCR = 0; // release all timers
	
    GTCCR = (1<<TSM) | (1<<PSRASY) | (1<<PSRSYNC);				//halt all timers

    //set TC1 to normal mode => TOV1 flag set at TOP = 0xFFFF
    TCCR1A &= 0x00; //reset TC1 control register A
    TCCR1B &= 0x00; //reset TC1 control register B

    //set TC1 to use external clock source, clocked on rising edge
	// the clock is equal to the frequency to measure
    TCCR1B |= (1<<CS10);
    TCCR1B |= (1<<CS11);
    TCCR1B |= (1<<CS12);

    //reset TC2
    TCCR2A &= 0x00; //reset TC2 control register A
    TCCR2B &= 0x00; //reset TC2 control register B

    //set TC2 to CTC Mode with TOP = OCR2A => counter cleared on reaching TOP
    TCCR2A |= (1<<WGM21);

    //TC2 is in CTC mode => set value of OCR2A = TOP
    OCR2A = 124;					//TOP is reached every 125 counts (starting from 0)

	//#if (F_CPU == 16000000UL)
	// at 16 MHz CPU
    //set TC2 to use a prescaler of 1 => 16MHz / 128 = 125kHz
    //TCCR2B |= (1<<CS20);
    //TCCR2B |= (1<<CS22);
	//#endif

	//#if (F_CPU == 8000000UL)	
	// at 8 MHz CPU
	//set TC2 to use a prescaler of 1 => 8MHz / 64 = 125kHz
	TCCR2B |= (1<<CS22);
	//#endif	

    f_ready = 0;					//reset ready flag
    f_tics = 0;						//reset TC2 interrupt counter

    //synchronize the value of both timers
    TCNT2 = 0;
    TCNT1 = 0;

    //clear interrupt flags
    TIFR2 |= (1<<OCF2A);
    TIFR1 |= (1<<TOV1);

    TIMSK2 |= (1<<OCIE2A);	//enable TC2 Output Compare A interrupt => triggered on reaching TOP = OCR2A
    TIMSK1 |= (1<<TOIE1);	//enable TC1 overflow interrupt => triggered on reaching TOP = MAX = 0xFFFF

    GTCCR = 0x00;			//restart all timers => start counting
	sei();
}

/*
 * This ISR is triggered by TC2 every 1ms because
 * TC2 increments with a frequency of 16MHz / 128 = 125KHz or 8MHz / 64 = 125kHz
 * and reaches TOP every 125 counts 
 * => trigger frequency = 1kHz
 * => trigger period = 1ms
 * 
 * This ISR handles the Gate Time generation
 */
ISR(TIMER2_COMPA_vect) 
{
    //increment number of TC2 interrupts (triggers when reaching TOP = OCR2A)
    FreqCounter::f_tics++;

    /*
     * If branch evaluates as true if given gate time (f_period) equals 
     * amount of milliseconds TC2 has counted (= f_tics)
     */
    if (FreqCounter::f_tics >= FreqCounter::f_period) 
    {
        //end of gate time => measurement ready
        TCCR1B &= ~0x07;					//set TC1 to no clk source => TC1 stops counting
        TIMSK2 &= ~(1<<OCIE2A);				//disable TC2 Output Compare A interrupt
        TIMSK1 &= ~(1<<TOIE1);				//disable TC1 overflow interrupt
        FreqCounter::f_ready = 1;			//set software flag for end of gating period

        //increment on overflow of TCNT1
        if ((TIFR1) & (1 << TOV1))			// Is Bit Set?
            FreqCounter::f_mlt++;

        //calculate frequency
        FreqCounter::f_freq = 65536 * FreqCounter::f_mlt;
        FreqCounter::f_freq += TCNT1;		//add value of TC1
        FreqCounter::f_freq = (long)(FreqCounter::f_freq * 1000.0 / (double)FreqCounter::f_tics);
        FreqCounter::f_mlt = 0;
    }
}

ISR(TIMER1_OVF_vect)
{
    FreqCounter::f_mlt++;
}

